(ns com.aurellem.assembly
  (:use (com.aurellem gb-driver vbm title items))
  (:import [com.aurellem.gb_driver SaveState]))

(defn mid-game []
  (read-state "mid-game"))

(defn inject-assembly
  [^SaveState state
   program-counter registers
   assembly-code]
  (let [scratch-memory (memory state)]
    ;; inject assembly code
    (dorun (map (fn [index val]
                  (aset scratch-memory index val))
                (range program-counter
                       (+ program-counter (count assembly-code)))
                assembly-code))
    (-> state
        (write-memory! scratch-memory)
        (write-registers! registers)
        (PC! program-counter))))
        

(defn inject-item-assembly
  ([^SaveState state assembly-code]
     (inject-assembly state (inc item-list-start)
                      (registers state)
                      assembly-code))
  ([assembly-code]
     (inject-item-assembly @current-state assembly-code)))

(defn info
  ([^SaveState state]
     (println "PC: " (PC state))
     (println "Instruction:"
              (format "0x%02X" (aget (memory state) (PC state))))
     state))    

(defn run-assembly
  ([info-fn assembly n]
     (let [final-state
           (reduce (fn [state _]
                     (tick (info-fn state)))
                   (inject-item-assembly (tick (tick (tick (mid-game)))) assembly)
                   (range n))]
       (shutdown!)
       final-state))
  ([assembly n]
     (run-assembly info assembly n)))
             